@@ -5,14 +5,14 @@ import string
 
 class generate_mac():
     '''MAC address generation class'''
-    _valid_char = "0123456789ABCDEF"
-    _valid_bcast_char = "02468ACE"
-    _valid_vendors = set()
+    _valid_char        = "0123456789ABCDEF"
+    _valid_bcast_char  = "02468ACE"
+    _valid_vendors     = set()
     _vid_table = {
     '36':2,
     '32':3,
     '28':4,
-    'none':6,
+    None:6,
     }
     def _strip_comments(in_list):
         '''Proccesses out comments using # as the comment character'''
@@ -28,11 +28,11 @@ class generate_mac():
         ''' Every line is [MAC,Vendor,Comment]'''
         out_lines = []
         try:
-            in_file    = open(vid_file,"r")
+            in_file    = open(vid_file,"r",encoding="utf8")
             file_lines = in_file.readlines()
             in_file.close()
         except FileNotFoundError:
-            raise FileNotFoundError("ERROR: Cannot read VID file " + in_file)
+            raise FileNotFoundError("Cannot read VID file " + vid_file)
 
         # Sanitize inputs
         file_lines = generate_mac._strip_comments(file_lines)
@@ -99,14 +99,21 @@ class generate_mac():
     def _is_byte(test_byte):
         '''Tests if a string is a hexdecimal byte, returns True/False'''
         test_byte = test_byte.strip(":")
+
+        # Bytes need to be two hexdecimal chars
+        if len(test_byte) != 2:
+            return False
+
         # If a byte does not convert from hex, its not a byte
         try:
             test_byte = int(test_byte,16)
         except:
             return False
+
         # If the value is out of range for a single byte, it is not
         if  0 < test_byte > 255:
             return False
+
         #If we get to the end, its a byte
         return True
 
@@ -138,25 +145,45 @@ class generate_mac():
         output = vid_bytes + ":" + dev_bytes
         return(output)
         
-    def vid_file_vendor(vid_file,vendor):
-        '''Generates a random MAC from a specified vendor name'''
+    def vid_file_vendor(vid_file,vendor,desc=None):
+        '''Generates a random MAC from a specified vendor name. Takes three parameters. file name, vendor name, and description(optional). description can be a partial match'''
         file_lines = generate_mac._read_vid_file(vid_file)
-        # Get a random VID assigned to Apple Inc
+
         line_vendor = ""
         vid_bytes   = ""
-        descrption  = ""
+        file_desc   = ""
         rand_line   = ""
-        if vendor not in generate_mac._valid_vendors:
-            raise KeyError(vendor + " has no associated VID byte in manuf file")
-        while line_vendor != vendor:
-            # Get a random line from the file, and then proccess
-            rand_line    = file_lines[ random.randrange( len(file_lines) ) ]
-            rand_line    = generate_mac._get_processed_vid(rand_line)
-            # Fill in the blanks
-            vid_bytes    = rand_line[0]
-            bytes_needed = rand_line[1]
-            line_vendor  = rand_line[2]
-            description  = rand_line[3]
+        # If the vendor is not in vendor list, throw an error, otherwise
+        # function will hang if vendor not in
+        # generate_mac._valid_vendors: raise KeyError(vendor + " has no
+        # associated VID byte in manuf file")
+
+        search_lines = []
+        for line in file_lines:
+            line = generate_mac._get_processed_vid(line)
+            # split the line into named variables. for formating see above fuction _get_processed_vid()
+            if len(line) == 4:
+                vid_bytes,bytes_needed,line_vendor,file_desc = line
+            elif len(line) == 3:
+                vid_bytes,bytes_needed,line_vendor = line
+            else:
+                continue
+
+            # Compile a list of matching lines into search_lines. use description if its present
+            if desc == None and vendor.lower() == line_vendor.lower():
+                search_lines.append(line)
+            elif desc == None:
+                continue
+            elif desc.lower() in file_desc.lower() and vendor.lower() == line_vendor.lower():
+                search_lines.append(line)
+            else:
+                continue
+
+        # If there are no search_lines, there are no matches, raise error
+        if len(search_lines) == 0:
+            raise KeyError("No Match Vendor: " + vendor + " Desc: " + str(desc))          
+        rand_line = search_lines[ random.randrange( len(search_lines) ) ]
+        vid_bytes,bytes_needed,line_vendor,file_desc = rand_line
 
         # Now generate the random device bytes
         bytes_needed = rand_line[1] //2
@@ -168,20 +195,23 @@ class generate_mac():
     def vid_provided(vid_bytes):
         '''Generates only the Device bytes, given specified VID bytes'''
         test_vid = []
-        i = ""
         rand_bytes = ""
         output = ""
         ## Start with error checking
         #remove any trailing :
-        vid_bytes = vid_bytes.rstrip(":")
-        test_vid = vid_bytes.split(":")
+        try:
+            vid_bytes = vid_bytes.rstrip(":")
+            test_vid = vid_bytes.split(":")
+        except:
+            raise ValueError(str(vid_bytes) + ' are not valid VID bytes')
         # If there aren't precisely three bytes, its not a VID
         if len(test_vid) != 3:
             raise ValueError(vid_bytes + ' are not valid VID bytes')
-        for i in test_vid:
-            if generate_mac._is_byte(i) == False:
-                raise ValueError(vid_bytes + ' are not valid VID bytes')
-        
+        for byte in test_vid:
+            if generate_mac._is_byte(byte) == False:
+                raise ValueError(str(vid_bytes) + ' are not valid VID bytes')
+
+        # generate some new device bytes        
         rand_bytes = generate_mac._gen_rand_bytes(3)
         output = vid_bytes + ":" + rand_bytes
         return output
@@ -189,10 +219,11 @@ class generate_mac():
     def list_vendors(vid_file):
         '''Returns a list[] of valid ETH Vendors that can be used with vid_file_vendor()'''
         file_lines = generate_mac._read_vid_file(vid_file)
-        return list(generate_mac._valid_vendors)
+        return generate_mac._valid_vendors
 
     def is_mac_address(mac_address):
         '''Test if a given string is a valid Ethernet MAC address. return True or False'''
+        # Check to make sure its 6 fields split with a colon - :
         try:
             mac_bytes=mac_address.split(":")
         except:
@@ -200,6 +231,11 @@ class generate_mac():
         if len(mac_bytes) != 6:
             return False
 
+        # Make sure all bytes are in fact bytes
+        for byte in mac_bytes:
+          if generate_mac._is_byte(byte) == False:
+              return False
+         
         # First Octet needs to be odd.       
         mac_byte_bcast = mac_bytes[0][1]
         mac_byte_bcast = mac_byte_bcast.upper()
@@ -207,3 +243,27 @@ class generate_mac():
             return True
         else:
             return False
+    
+    def get_vid_bytes(mac_address):
+        '''Return vendor bytes from a given MAC address as a string'''
+        # check if this is a valid mac address
+        if generate_mac.is_mac_address(mac_address) != True:
+            raise ValueError(str(mac_address) + ' is not a valid MAC address')
+
+        output = ""
+        # Grab the first three bytes, this is the VID
+        mac_bytes = mac_address.split(":")
+        output = ":".join(mac_bytes[0:3])
+
+        return output
+    
+    def another_same_vid(mac_address):
+        '''Generate another MAC address from the same VID bytes'''
+        output = ""
+            
+        vid_bytes = generate_mac.get_vid_bytes(mac_address)
+        dev_bytes = generate_mac._gen_rand_bytes(3)
+        output    = vid_bytes + ":" + dev_bytes
+        output    = output.upper()
+
+        return output
